/*
 * CustomerServiceMock.java
 */

package com.example.business;
import java.util.List;
import com.example.integration.CustomerDAO;

/**
 * Mock object for interface CustomerService.
 */
public class CustomerServiceImpl implements CustomerService {
    
    /**
     * Find customers based on the given criteria.
     * @param criteria find criteria.
     * @return the list of CustomerBO objects.
     */
    public List findCustomer(CustomerCriteria criteria) {
        return customerDAO.find(criteria.getName(),
                                 criteria.getCompany());
    }
    
    /**
     * Delete customer which has the specified id.
     * 
     * @param id the customer id.
     * @param criteria find criteria.
     * @return the list of CustomerBO objects.
     */
    public List deleteCustomer(int id, CustomerCriteria criteria) {
        customerDAO.delete(id);
        return findCustomer(criteria);
    }

    /**
     * Holds value of property customerDAO.
     */
    private CustomerDAO customerDAO;
    
    /**
     * Getter for property customerDAO.
     * @return Value of property customerDAO.
     */
    public CustomerDAO getCustomerDAO() {
        return this.customerDAO;
    }
    
    /**
     * Setter for property customerDAO.
     * @param customerDAO New value of property customerDAO.
     */
    public void setCustomerDAO(CustomerDAO customerDAO) {
        this.customerDAO = customerDAO;
    }    
}
