/*
 * InvocationContext.java
 */

package com.example.interceptor;

import java.lang.reflect.Method;
import java.util.Map;
import javax.ejb.EJBContext;

/**
 * Invocation context interface that is defined in EJB 3.0 draft specification.
 */
public interface InvocationContext {

    /**
     * Obtains the target object.
     * @return target object.
     */
    Object getBean();

    /**
     * Obtains the target method.
     * @return target method.
     */
    Method getMethod();

    /**
     * Obtains the parameter list of the target method.
     * @return parameter list.
     */
    Object[] getParameters();

    /**
     * Sets the modified parameter list before method invocation.
     * @param args method parameter list.
     */
    void setParameters(Object[] args);

    /**
     * Obtains EJB context.
     * @return EJB context.
     */
    EJBContext getEJBContext();

    /**
     * Obtains context data.
     * @return context data.
     */
    Map getContextData();

    /**
     * Invokes the target method.
     * @return the result of the method invocation.
     * @throws Exception thrown if the target method invocation failed.
     */
    Object proceed() throws Exception;
}
