/*
 * SecurityInterceptor.java
 */

package com.example.interceptor.security;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import com.example.util.ContextUtil;

/**
 * Security interceptor for the annotation type RolesAllowed.
 */
public class SecurityInterceptor implements MethodInterceptor {
    /**
     * Check whether method invocation is allowed by annotated security role.
     * @param inv method invocation context.
     * @return the result of method invocation.
     * @throws Throwable thrown if the target method invocation failed.
     */
    public Object invoke(MethodInvocation inv) throws Throwable {
        RolesAllowed annotation =
                inv.getMethod().getAnnotation(RolesAllowed.class);
        if (annotation != null) {
            String roles[] = annotation.value();
            boolean allowed = false;
            for (int i = 0; i < roles.length; i++) {
                if (ContextUtil.isUserInRole(roles[i])) {
                    allowed = true;
                    break;
                }
            }
            if (!allowed) {
                throw new SecurityException(
                        "Client not authorized for this invocation.");
            }
        }
        return inv.proceed();
    }
}
