/*
 * Tracer.java
 */

package com.example.interceptor.trace;

import java.security.Principal;
import java.util.logging.Level;
import java.util.logging.Logger;
import com.example.interceptor.InvocationContext;
import com.example.util.ContextUtil;

/**
 * Tracer interceptor for Java EE 5.
 */
public class Tracer {
    /**
     * Log message resource
     */
    private static final String RESOURCE_BUNDLE_NAME = "Tracer";
    
    /**
     * Message key for before log.
     */
    private static final String BEFORE_FORMAT_KEY = "tracer.before";
    
    /**
     * Message key for after log.
     */
    private static final String AFTER_FORMAT_KEY = "tracer.after";
    
    /**
     * Logger.
     */
    private static Logger log = Logger.getLogger(Tracer.class.getName(),
            RESOURCE_BUNDLE_NAME);
    
    /**
     * Outputs trace log before and after method invocations.
     * @param inv method invocation context.
     * @return the result of method invocation.
     * @throws Exception thrown if method invocation failed.
     */
    public Object trace(InvocationContext inv)
    throws Exception {
        Principal userPrincipal = ContextUtil.getUserPrincipal();
        String user = userPrincipal == null ? null : userPrincipal.getName();
        String className = inv.getBean().getClass().getName();
        String methodName = inv.getMethod().getName();
        String params = getParametersAsString(inv.getParameters());
        
        // Before Advise.
        log.log(Level.INFO, BEFORE_FORMAT_KEY,
                new Object[] {user, className, methodName, params});
                
        Object result = inv.proceed();
        
        // After Advice.
        log.log(Level.INFO, AFTER_FORMAT_KEY,
                new Object[] {user, className, methodName, params, result});
        return result;
    }
    
    /**
     * Returns string representation of parameters.
     * @param args the array of method parameter.
     * @return string representation of parameter list.
     */
    private String getParametersAsString(Object[] args) {
        if (args == null) {
            return "";
        }
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < args.length;) {
            buf.append(args[i]);
            if (++i != args.length) {
                buf.append(", ");
            }
        }
        return buf.toString();
    }
}
