/*
 * ClassGenerator.java
 */

package com.example.tool;
import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.MethodDoc;
import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;
import com.example.util.ResourceUtil;



/**
 * Generate a class source from a ClassDoc object.
 */
class ClassGenerator {
    
    /** Class docuement object. */
    private ClassDoc classdoc;
    /** Doclet options. */
    private Options options;
    /** Source code encoding */
    private static String ENCODING = "UTF-8";
    /** Class template file name */
    private static String TEMPLATE_NAME = "CreateSessionBean_class.template";
    /** Session bean class name suffix */
    private static String EJB_SUFFIX = "SessionBean";
    /** Class template */
    private static String template;
    
    /**
     * Creates a new instance of ClassGenerator
     * @param classdoc ClassDoc object.
     * @param options doclet options.
     */
    public ClassGenerator(ClassDoc classdoc, Options options)
    throws IOException {
        this.classdoc = classdoc;
        this.options = options;
        if (template == null) {
            template = ResourceUtil.getResourceAsString(this.getClass(),
                    TEMPLATE_NAME, ENCODING);
        }
    }
        
    /**
     * Generates a class source.
     * @return generated source code.
     * @throws IOException thrown if I/O exception occurs.
     */
    public String generate() throws IOException {
        String className = classdoc.name();
        String packageName = classdoc.containingPackage().name();
        String serviceName = className;
        String contextPath = options.getContext();
        String jndiPrefix = "ejb/";
        String ejbSuffix = EJB_SUFFIX;

        String gensrc = template;
        gensrc = gensrc.replaceAll("%CLASS_NAME%", className);
        gensrc = gensrc.replaceAll("%PACKAGE_NAME%", packageName);
        gensrc = gensrc.replaceAll("%SERVICE_NAME%", serviceName);
        gensrc = gensrc.replaceAll("%CONTEXT_PATH%", contextPath);
        gensrc = gensrc.replaceAll("%JNDI_PREFIX%", jndiPrefix);
        gensrc = gensrc.replaceAll("%EJB_SUFFIX%", ejbSuffix);

        // メソッド一覧の取得
        MethodDoc methoddoc[] = classdoc.methods();
        
        StringBuffer genmethods = new StringBuffer();
        for (int i = 0; i < methoddoc.length; i++) {
            MethodGenerator generator = new MethodGenerator(methoddoc[i]);
            genmethods.append(generator.generate());
        }

        gensrc = gensrc.replaceAll("%METHOD_DEFINITIONS%",
                genmethods.toString());
        return gensrc;
    }

    /**
     * Returns package path for this Class.
     * @return path representation of package name.
     */
    public String packagePath() {
        String packageName = classdoc.containingPackage().name();
        StringTokenizer tokens = new StringTokenizer(packageName, ".");
        StringBuffer path = new StringBuffer();
        while (tokens.hasMoreTokens()) {
            path.append(tokens.nextToken());
            if (tokens.hasMoreTokens()) {
                path.append(File.separator);                
            }
        }
        return path.toString();
    }
    
    /**
     * Returns generating class name
     * @return generating class name
     */
    public String generatedClassName() {
        return classdoc.name() + EJB_SUFFIX;
    }
}
