/*
 * CreateSessionBeanDoclet.java
 */

package com.example.tool;

import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.DocErrorReporter;
import com.sun.javadoc.RootDoc;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Custom doclet that creates session bean wrapper classes from the specified
 * service interfaces.
 */
public class CreateSessionBeanDoclet {
    
    /** Doclet options */
    private static Options options = new Options();
    /** Output encoding */
    private static String OUTPUT_ENCODING = "UTF-8";
    
    /**
     * Start point of this doclet.
     * @param root RootDoc object.
     * @return true if succeeded, otherwise false.
     * @throws Exception thrown if doclet failed.
     */
    public static boolean start(RootDoc root) throws Exception {
        
        // クラス一覧の取得
        ClassDoc[] classdocs = root.classes();
        
        for(int i = 0; i < classdocs.length; i++){
            // EJBクラスの生成
            ClassGenerator generator =
                    new ClassGenerator(classdocs[i], options);
            String gensrc = generator.generate();
            // EJBクラスをファイルに保存
            saveStringAsFile(gensrc,
                    generator.packagePath(),
                    generator.generatedClassName());
        }
        return true;
    }
    
    /**
     * Save the generated source code to the specified file.
     * @param gensrc generated source code.
     * @packagePath the relative path of package name.
     * @className class name.
     * @throws IOException thrown if I/O exception occurs.
     */
    private static void saveStringAsFile(String gensrc,
            String packagePath, String className) throws IOException {
        String path = options.getDir() + File.separator + packagePath;
        mkdir(path);
        String filePath = path + File.separator + className + ".java";
        OutputStream os =
                new BufferedOutputStream(new FileOutputStream(filePath));
        os.write(gensrc.getBytes(OUTPUT_ENCODING));
        os.close();
    }
    
    /**
     * Make directory of the specified path.
     * @param path directory name.
     * @return true if the directory created.
     */
    private static boolean mkdir(String path) {
        File newdir =new File(path);
        return newdir.mkdirs();
    }
    
    /**
     * Returns option length specification.
     * @param option doclet option.
     * @return the length of this option.
     */
    public static int optionLength(String option) {
        if(option.equals("-dir") ||
                option.equals("-context")) {
            return 2;
        }
        return 0;
    }
    
    /**
     * Validates the given options of this doclet.
     * @param opts the array of options.
     * @param reporter doclet error reporter.
     * @return true if doclet succeeded.
     */
    public static boolean validOptions(String opts[][],
            DocErrorReporter reporter) {
        for (int i = 0; i < opts.length; i++) {
            String[] opt = opts[i];
            if (opt[0].equals("-dir")) {
                options.setDir(opt[1]);
            } else if (opt[0].equals("-context")) {
                options.setContext(opt[1]);
            } else if (opt[0].equals("-doclet")) {
                continue;
            } else if (opt[0].equals("-docletpath")) {
                continue;
            } else if (opt[0].equals("-encoding")) {
                continue;
            } else if (opt[0].equals("-verbose")) {
                continue;
            } else if (opt[0].equals("-classpath")) {
                continue;
            } else {
                reporter.printError("Option " + opt[0]
                        + " is not supported.\n"
                        + usage);
                return false;
            }
        }
        return true;
    }
    
    /** Usage string */
    private static String usage =
            "Usage: javadoc [-dir gen_dir] [-context context_path] -doclet "
            + com.example.tool.CreateSessionBeanDoclet.class.getName()
            + " ...";
    
}
