/*
 * ListBean.java
 */

package com.example.web;

import java.util.List;
import java.util.Map;
import java.util.logging.Logger;
import java.util.logging.Level;
import javax.faces.event.ActionEvent;
import javax.faces.context.FacesContext;
import com.example.business.CustomerCriteria;
import com.example.business.CustomerService;
import com.example.model.CustomerBO;

/**
 * ListBean: JSF backing bean.
 */
public class ListBean {
    
    /** Logger */
    private static Logger log = Logger.getLogger(ListBean.class.getName());
    
    /**
     * Action method to find customers.
     */
    public String find() {
        try {
            customerList = customerService.findCustomer(criteria);
            return "success";
        } catch (Exception e) {
            return handleError(e);
        }
    }
    
    /**
     * Action method to delete customer.
     */
    public String delete() {
        try {
            // 選択されたCustomerBOを取得する
            FacesContext context = FacesContext.getCurrentInstance();
            Map requestMap = context.getExternalContext().getRequestMap();
            CustomerBO cust = (CustomerBO)requestMap.get("customer");
            // 選択されたCustomerBOを削除する
            int id = cust.getId();
            customerList = customerService.deleteCustomer(id, criteria);
            return "success";
        } catch (Exception e) {
            return handleError(e);
        }
    }
    
    /**
     * Action method to select a customer.
     */
    public String select() {
        // 選択されたCustomerBOを取得する
        FacesContext context = FacesContext.getCurrentInstance();
        Map requestMap = context.getExternalContext().getRequestMap();
        CustomerBO cust = (CustomerBO)requestMap.get("customer");
        // 選択されたcustをDetailsBeanに設定する
        context.getApplication()
        .createValueBinding("#{detailsBean.customer}")
        .setValue(context, cust);
        // #{detailsBean}をsessionスコープにすれば以下でもよい．
        //detailsBean.setCustomer(cust);
        return "success";
    }

    /** Fail-safe for tomcat. */
    private static Class accessLocalExceptionClass;
    static {
        try {
            accessLocalExceptionClass = 
                    Class.forName("javax.ejb.AccessLocalException");
        }
        catch (ClassNotFoundException e) {}
    }
    
    /**
     * Handles exceptions and determines navigation page id.
     */
    public String handleError(Exception e) {
        if (e instanceof SecurityException) {
            log.log(Level.INFO, "Access violation occurs.", e);
            return "error.security";
        }
        else if (accessLocalExceptionClass != null &&
                accessLocalExceptionClass.isInstance(e)) {
            log.log(Level.INFO, "Access violation occurs.", e);
            return "error.security";            
        }
        else {
            log.log(Level.WARNING, "System error occurs.", e);
            return "error";
        }
    }
    
    /**
     * Holds value of property criteria.
     */
    private CustomerCriteria criteria;
    
    /**
     * Getter for property criteria.
     * @return Value of property criteria.
     */
    public CustomerCriteria getCriteria() {
        return this.criteria;
    }
    
    /**
     * Setter for property criteria.
     * @param criteria New value of property criteria.
     */
    public void setCriteria(CustomerCriteria criteria) {
        this.criteria = criteria;
    }
    
    /**
     * Holds value of property customerService.
     */
    private CustomerService customerService;
    
    /**
     * Getter for property customerService.
     * @return Value of property customerService.
     */
    public CustomerService getCustomerService() {
        return this.customerService;
    }
    
    /**
     * Setter for property customerService.
     * @param customerService New value of property customerService.
     */
    public void setCustomerService(CustomerService customerService) {
        this.customerService = customerService;
    }
    
    /**
     * Action event listener for clear criteria.
     */
    public void clear(ActionEvent ev) {
        criteria.clear();
    }
    
    /**
     * Holds value of property customerList.
     */
    private List customerList;
    
    /**
     * Getter for property customerList.
     * @return Value of property customerList.
     */
    public List getCustomerList() {
        return this.customerList;
    }
    
    /**
     * Holds value of property detailsBean.
     */
    private DetailsBean detailsBean;
    
    /**
     * Getter for property detailsBean.
     * @return Value of property detailsBean.
     */
    public DetailsBean getDetailsBean() {
        
        return this.detailsBean;
    }
    
    /**
     * Setter for property detailsBean.
     * @param detailsBean New value of property detailsBean.
     */
    public void setDetailsBean(DetailsBean detailsBean) {
        
        this.detailsBean = detailsBean;
    }
    
}
